package com.yihu.jw.hospital.endpoint.statistics;


import com.alibaba.fastjson.JSONObject;
import com.yihu.jw.es.service.StatisticsEsService;
import com.yihu.jw.restmodel.web.ObjEnvelop;
import com.yihu.jw.restmodel.web.endpoint.EnvelopRestEndpoint;
import com.yihu.jw.rm.hospital.BaseHospitalRequestMapping;
import com.yihu.jw.utils.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * Created by wangzhinan on 2019/12/3.
 */
@RestController
@RequestMapping(value = BaseHospitalRequestMapping.Statistics.PREFIX)
@Api(value = "统计分析", description = "统计分析", tags = {"统计分析"})
public class EsStatisticsEndpoint extends EnvelopRestEndpoint {

    Logger logger = LoggerFactory.getLogger(EsStatisticsEndpoint.class);



    @Autowired
    private StatisticsEsService statisticsEsService;

    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientLine)
    @ApiOperation(value = "门诊类别统计-曲线")
    public ObjEnvelop getOutPatientLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {
            //新版与旧版统计适配
        JSONObject result = new JSONObject();
        logger.info("index:"+index+" and level2_type:"+level2_type+" and level:"+level);
        try {
            if (index.equalsIgnoreCase("3,21")||index.equalsIgnoreCase("3")||index.equalsIgnoreCase("21")){
                result= statisticsEsService.getOutPatientPriceLine(startDate, endDate, area, level, index, interval,level2_type);
            }else {
                result= statisticsEsService.getOutPatientLine(startDate, endDate, area, level, index, interval,level2_type);
            }
            return success(result);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }


    }


    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.prescriptionLine)
    @ApiOperation(value = "开方量周统计-曲线")
    public ObjEnvelop getPrescriptionLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getPrescriptionLine(startDate, endDate, area, level, index, interval,level2_type);
            return success(result);
        } catch (Exception e) {
            return failedObjEnvelopException(e);
        }

    }
    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.prescriptionTj)
    @ApiOperation(value = "开方量统计表格")
    public ObjEnvelop prescriptionTj(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getPrescriptionLineBy(startDate, endDate, area, level, index, interval,level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }
    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.prescriptionByKd)
    @ApiOperation(value = "开方快递量统计表格")
    public ObjEnvelop prescriptionByKd(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getPrescriptionLineByType(startDate, endDate, area, level, index, interval,level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.total)
    @ApiOperation(value = " 查询总数")
    public ObjEnvelop getIndexTotal(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getTotalAmount(startDate, endDate, area, level, index, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientCircular)
    @ApiOperation(value = "门诊类别统计-圆形")
    public ObjEnvelop getOutPatientCircular(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getOutPatientCircular(startDate, endDate, area, level, index, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * 门诊统计-分级
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientGrade)
    @ApiOperation(value = "门诊统计-分级")
    public ObjEnvelop getOutPatient(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getOutPatientGrade(startDate, endDate, area, level, index, level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 问诊量统计-类别曲线
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param index
     * @param level2_type 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.outPatientTypeLine)
    @ApiOperation(value = "问诊量统计-类别曲线")
    public ObjEnvelop getOutPatientTypeline(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String level2_type) {
        String[] indexes = index.split(",");
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getOutPatientTypeline(startDate, endDate, area, level, index,level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.doctorStatictis)
    @ApiOperation(value = "人员统计")
    public ObjEnvelop getDoctorStatistics(@RequestParam(required = true) String startDate,
                                          @RequestParam(required = true) String endDate,
                                          @RequestParam(required = true) String area,
                                          @RequestParam(required = true) int level,
                                          @RequestParam(required = true) String index,
                                          @RequestParam(required = false) String level2_type,
                                          @RequestParam(required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getDoctorStatistics(startDate,endDate,area,level,index,level2_type,flag);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.doctorTotal)
    @ApiOperation(value = "各职称人数统计")
    public ObjEnvelop getDoctorTotal(@RequestParam(required = true) String startDate,
                                     @RequestParam(required = true) String endDate,
                                     @RequestParam(required = true) String area,
                                     @RequestParam(required = true) int level,
                                     @RequestParam(required = true) String index,
                                     @RequestParam(required = false) String level2_type,
                                     @RequestParam(required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getDoctorTotal(startDate,endDate,area,level,index,level2_type,flag);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.doctorStatictisExcel)
    @ApiOperation(value = "人数统计-excel表")
    public ObjEnvelop getDoctorStatisticsExcel(@RequestParam(required = true) String startDate,
                                     @RequestParam(required = true) String endDate,
                                     @RequestParam(required = true) String area,
                                     @RequestParam(required = true) int level,
                                     @RequestParam(required = true) String index,
                                     @RequestParam(required = false) String level2_type,
                                     @RequestParam(required = true) Integer flag) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getDoctorStatisticsExcel(startDate,endDate,area,level,index,level2_type,flag);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 专科咨询列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.specialConsultHead)
    @ApiOperation(value = " 专科咨询分析-头部")
    public ObjEnvelop specialConsultHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getSpecialConsultHead(startDate, endDate, area, level, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 专科咨询列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.specialConsultLine)
    @ApiOperation(value = "专科咨询分析曲线")
    public ObjEnvelop specialConsultLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getSpecialConsultLine(startDate, endDate, area, level, index, interval,level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 专科咨询列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param lowLevel 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.specialConsultExcel)
    @ApiOperation(value = "专科咨询列表")
    public ObjEnvelop specialConsultExcel(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String lowLevel) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getConsultingStatisticsList(startDate, endDate,level,area,lowLevel);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }



    /**
     * 服务评价总平均分
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scoreHead)
    @ApiOperation(value = " 服务评价总平均分-头部")
    public ObjEnvelop getScoreHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        //新版与旧版统计适配
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getScoreHead(startDate, endDate, area, level, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 服务评价分析曲线
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scoreLine)
    @ApiOperation(value = "服务评价分析曲线")
    public ObjEnvelop getScoreLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getScoreLine(startDate, endDate, area, level, index, interval,level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 服务评价分析列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param lowLevel 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scoreList)
    @ApiOperation(value = "服务评价分析列表")
    public ObjEnvelop getScoreList(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String lowLevel) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getScoreList(startDate, endDate,level,area,lowLevel);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scheduleHead)
    @ApiOperation(value = "排班头部")
    public ObjEnvelop getScheduleTotal(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getScheduleTotal(startDate, endDate,index,level,area);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scheduleLine)
    @ApiOperation(value = "排班曲线")
    public ObjEnvelop getScheduleLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true)Integer interval) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getScheduleLine(startDate, endDate,area,level,index,interval);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.scheduleList)
    @ApiOperation(value = "排班表格")
    public ObjEnvelop getScheduleList(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true)String levelCode) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getScheduleList(startDate, endDate,index,level,area,levelCode);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 咨询列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultHead)
    @ApiOperation(value = " 咨询分析-头部")
    public ObjEnvelop getConsultHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String lowCode) {
        //新版与旧版统计适配
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultHead(startDate, endDate, area, level, index,lowCode);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * 咨询列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultLine)
    @ApiOperation(value = "咨询分析曲线")
    public ObjEnvelop getConsultLine(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false)int interval,
            @RequestParam(required = false) String level2_type) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getConsultLine(startDate, endDate, area, level, index, interval,level2_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * 咨询列表
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5、科室 6医生
     * @param lowLevel 2 市  3区  4医院 5、科室 6医生
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultList)
    @ApiOperation(value = "咨询列表")
    public ObjEnvelop getConsultingList(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true) String lowLevel) {

        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.getConsultingList(startDate, endDate,level,area,lowLevel,index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * app端首页门诊量统计
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index 5咨询量，6专家咨询,11图文复诊，13专科协同，15视频复诊
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultTotalApp)
    @ApiOperation(value = "app端首页门诊量统计")
    public ObjEnvelop getConsultTotalApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultTotalApp(startDate, endDate, area, level, index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * app端首页门诊量-圆形图
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultCircularApp)
    @ApiOperation(value = "app端首页门诊量-圆形图")
    public ObjEnvelop getConsultCircularApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultCircularApp(startDate, endDate, area, level);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     * app端首页门诊量统计-曲线图
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index 5
     * @param interval
     * @param level2_type
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultLineApp)
    @ApiOperation(value = "app端首页门诊量统计-曲线图")
    public ObjEnvelop getConsultLineApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true)Integer interval,
            @RequestParam(required = false)String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultLineApp(startDate, endDate, area, level, index,interval,level2_type);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultListApp)
    @ApiOperation(value = "app端首页门诊量统计-表格")
    public ObjEnvelop getConsultListApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true)String lowLevel) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultListApp(startDate, endDate,level,area, lowLevel,index);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index 专家咨询6，图文复诊11，专家协同13，视频复诊15
     * @param interval
     * @param level2_type
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultLinePC)
    @ApiOperation(value = "pc端首页门诊量统计-曲线图")
    public ObjEnvelop getConsultLinePC(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true)Integer interval,
            @RequestParam(required = false)String  lowCode,
            @RequestParam(required = false)String level2_type) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultLinePC(startDate, endDate, area, level, index,interval,level2_type,lowCode);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     *pc端首页门诊量统计-表格
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index 专家咨询6，图文复诊11，专家协同13，视频复诊15
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.consultListPC)
    @ApiOperation(value = "pc端首页门诊量统计-表格")
    public ObjEnvelop getConsultLinePC(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String lowCode,
            @RequestParam(required = true)String lowLevel) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getConsultingListPC(startDate, endDate, level,area,lowLevel, index,lowCode);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * app端首页数据统计
     * @param startDate
     * @param endDate
     * @param area
     * @param level
     * @param index 6专家，13协同，11、在线复诊
     * @param timeLevel
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.indexTotalApp)
    @ApiOperation(value = "app端首页数据统计")
    public ObjEnvelop getIndexTotalApp(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = true)String timeLevel) {
        JSONObject result = new JSONObject();
        try {
            result = statisticsEsService.getIndexTotalApp(startDate, endDate,area, level,index,timeLevel);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }


    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appIndexHead)
    @ApiOperation(value = "app首页头部收入统计")
    public ObjEnvelop appIndexHead(
            @RequestParam(required = false) String startDate,
            @RequestParam(required = false) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.appIndexHead(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }

    /**
     *
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appIndexCenter)
    @ApiOperation(value = "app首页中部门诊量统计")
    public ObjEnvelop appIndexCenter(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = false) String index) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.appIndexCenter(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * app收入统计-头部
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appPriceHead)
    @ApiOperation(value = "app收入统计-头部")
    public ObjEnvelop appPriceHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.appPriceHead(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return success(result);
    }


    /**
     * app收入统计-头部
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appPrescriptionHead)
    @ApiOperation(value = "app处方统计-头部")
    public ObjEnvelop appPrescriptionHead(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.appPrescriptionHead(startDate, endDate, area, level, index);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }

    /**
     * app中间饼状图-中部
     * @param startDate
     * @param endDate
     * @param area
     * @param level 2 市  3区  4医院 5科室 6医生
     * @param index
     * @return
     */
    @GetMapping(value = BaseHospitalRequestMapping.Statistics.appCenterCircular)
    @ApiOperation(value = "app中间饼状图-中部")
    public ObjEnvelop appCenter(
            @RequestParam(required = true) String startDate,
            @RequestParam(required = true) String endDate,
            @RequestParam(required = true) String area,
            @RequestParam(required = true) int level,
            @RequestParam(required = true) String index,
            @RequestParam(required = false) String flag) {
        JSONObject result = new JSONObject();
        try {
            result= statisticsEsService.appCenterCircular(startDate, endDate, area, level, index,flag);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(result);
    }
}
